﻿Type=Class
Version=4.2
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Custom View class
#Event: LayoutChanged
Sub Class_Globals
	Private fx As JFX
	Private EventName As String 'ignore
	Private CallBack As Object 'ignore
	Private mBase As Pane
	Private CurrentPrinter As Printer
	Private CurrentPaper As Paper
	Private CurrentOrientation  As String
	Private CurrentUnits As String = "in"
	
	Private PrinterMap As Map
	Private PaperMap As Map
	
	Private cbPrinter As ComboBox
	Private cbPaper As ComboBox
	Private nsLeft As NumberSpinner
	Private nsTop As NumberSpinner
	Private nsRight As NumberSpinner
	Private nsBottom As NumberSpinner
	Private pnMargins As Pane
	Private pnNSBottom As Pane
	Private pnNSLeft As Pane
	Private pnNSRight As Pane
	Private pnNSTop As Pane
	Private cbUnits As ChoiceBox
	Private rbLandscape As RadioButton
	Private rbPortrait As RadioButton
	Private SettingUp As Boolean
End Sub

Public Sub Initialize (vCallback As Object, vEventName As String)
	EventName = vEventName
	CallBack = vCallback
	PrinterMap.Initialize
	PaperMap.Initialize
	SettingUp = True
End Sub

Public Sub DesignerCreateView (Base As Pane, Lbl As Label, Props As Map)
	mBase = Base
	
	mBase.LoadLayout("PrinterSetup")
	
	nsLeft.Initialize(Me,"NSLeft",1,2,.01)
	nsTop.Initialize(Me,"NSTop",1,2,.01)
	nsRight.Initialize(Me,"NSRight",1,2,.01)
	nsBottom.Initialize(Me,"NSBottom",1,2,.01)
	
	pnMargins.AddNode(nsLeft.AsNode,pnNSLeft.Left,pnNSLeft.Top,pnNSLeft.PrefWidth,pnNSLeft.PrefHeight)
	pnMargins.AddNode(nsTop.AsNode,pnNSTop.Left,pnNSTop.Top,pnNSTop.PrefWidth,pnNSTop.PrefHeight)
	pnMargins.AddNode(nsRight.AsNode,pnNSRight.Left,pnNSRight.Top,pnNSRight.PrefWidth,pnNSRight.PrefHeight)
	pnMargins.AddNode(nsBottom.AsNode,pnNSBottom.Left,pnNSBottom.Top,pnNSBottom.PrefWidth,pnNSBottom.PrefHeight)
	
	pnNSLeft.RemoveNodeFromParent
	pnNSTop.RemoveNodeFromParent
	pnNSRight.RemoveNodeFromParent
	pnNSBottom.RemoveNodeFromParent
	
	cbUnits.Items.AddAll(Array As String("in","pt","mm"))
	cbUnits.SelectedIndex = 0
	
	
	CurrentPrinter = Printer_Static.GetDefaultPrinter
	CurrentPaper = CurrentPrinter.GetDefaultPageLayout.GetPaper
	
	nsLeft.Value = ConvertMeasure(CurrentPrinter.GetDefaultPageLayout.GetLeftMargin,"ptin")
	nsTop.Value = ConvertMeasure(CurrentPrinter.GetDefaultPageLayout.GetTopMargin,"ptin")
	nsRight.Value = ConvertMeasure(CurrentPrinter.GetDefaultPageLayout.GetRightMargin,"ptin")
	nsBottom.Value = ConvertMeasure(CurrentPrinter.GetDefaultPageLayout.GetBottomMargin,"ptin")
	
	For Each Pr As Printer In Printer_Static.GetAllPrinters
		cbPrinter.Items.Add(Pr.GetName)
		PrinterMap.Put(Pr.GetName,Pr)
	Next
	
	cbPrinter.Value = CurrentPrinter.GetName
	
	rbLandscape.GroupRadioButtons(Array As Object(rbLandscape,rbPortrait))
	rbPortrait.Selected = True					'Default
	CurrentOrientation = "PORTRAIT"
	
	CallSubDelayed(Me,"SetupComplete")
End Sub

Private Sub SetupComplete
	SettingUp = False
End Sub

Private Sub Base_Resize (Width As Double, Height As Double)
	
End Sub

Public Sub GetBase As Pane
	Return mBase
End Sub


Private Sub cbPrinter_ValueChanged (Value As Object)
	CurrentPrinter = PrinterMap.Get(Value)

	cbPaper.Items.Clear
	PaperMap.clear
	For Each O As Object In CurrentPrinter.GetPrinterAttributes.GetSupportedPapers
		Dim PA As Paper
		PA.Initialize
		PA.SetObject(O)
		cbPaper.Items.Add(PA.GetName)
		PaperMap.Put(PA.GetName,PA)
	Next
	
	'Is the currently selected paper available?
'	If cbPaper.Items.IndexOf(CurrentPaper.GetName) > -1 Then
'		cbPaper.Value = CurrentPaper.GetName
'	Else
'		'Reset to default for the printer
		CurrentPaper = PaperMap.GetDefault(CurrentPrinter.GetDefaultPageLayout.GetPaper.GetName,CurrentPrinter.GetDefaultPageLayout.GetPaper)
		cbPaper.Value = CurrentPaper.GetName
'	End If
	LayoutChanged
End Sub

Private Sub cbPaper_ValueChanged (Value As Object)
	CurrentPaper = PaperMap.Get(Value)
	LayoutChanged
End Sub

Private Sub cbUnits_SelectedIndexChanged(Index As Int, Value As Object)
	nsLeft.Value = ConvertMeasure(nsLeft.Value,CurrentUnits & Value)
	nsTop.Value = ConvertMeasure(nsTop.Value,CurrentUnits & Value)
	nsRight.Value = ConvertMeasure(nsRight.Value,CurrentUnits & Value)
	nsBottom.Value = ConvertMeasure(nsBottom.Value,CurrentUnits & Value)
	CurrentUnits = Value
End Sub

Private Sub ConvertMeasure(Val As Double,Convert As String) As Double
	Select Convert.ToLowerCase
		Case "ptin"
			Return Val / 72
		Case "inpt"
			Return Val * 72
		Case "inmm"
			Return Val * 25.4
		Case "mmin"
			Return Val / 25.4
		Case "ptmm"
			Return (Val / 72) * 25.4
		Case "mmpt"
			Return(Val / 25.4) * 72
	End Select
	Return Val
End Sub

Private Sub LayoutChanged
	If SettingUp Then Return
	If SubExists(CallBack,EventName & "_LayoutChanged") Then CallSub(CallBack,EventName & "_LayoutChanged")
End Sub

Private Sub Orientation_SelectedChange(Selected As Boolean)
	If Selected Then
		Dim RB As RadioButton = Sender
		Dim Found As Boolean
		
		For Each S As String In CurrentPrinter.GetPrinterAttributes.GetSupportedPageOrientations
			If S = RB.Tag Then 
				Found = True
				Exit
			End If
		Next
		
		If Found Then
			CurrentOrientation = RB.Tag
		Else
			CurrentOrientation = CurrentPrinter.GetPrinterAttributes.GetDefaultPageOrientation
			If CurrentOrientation = "LANDSCAPE" Then
				rbLandscape.Selected = True
			Else
				rbPortrait.Selected = True
			End If
		End If
		LayoutChanged
	End If
End Sub

Sub getLeftMargin As Int
	Return ConvertMeasure(nsLeft.Value,CurrentUnits & "pt")
End Sub
Sub getTopMargin As Int
	Return ConvertMeasure(nsTop.Value,CurrentUnits & "pt")
End Sub
Sub getRightMargin As Int
	Return ConvertMeasure(nsRight.Value,CurrentUnits & "pt")
End Sub
Sub getBottomMargin As Int
	Return ConvertMeasure(nsBottom.Value,CurrentUnits & "pt")
End Sub

Sub getOrientation As String
	Return CurrentOrientation
End Sub
Sub getPrinter As Printer
	Return CurrentPrinter
End Sub
Sub getPaper As Paper
	Return CurrentPaper
End Sub